@@ -14,6 +14,7 @@
 
 #include <json.h>
 
+#include "cleanup.h"
 #include "fabrics.h"
 #include "log.h"
 #include "private.h"
@@ -59,6 +60,8 @@ static void json_update_attributes(nvme_ctrl_t c,
 					data_digest, val_obj);
 		JSON_UPDATE_BOOL_OPTION(cfg, key_str,
 					tls, val_obj);
+		JSON_UPDATE_BOOL_OPTION(cfg, key_str,
+					concat, val_obj);
 		if (!strcmp("persistent", key_str) &&
 		    !nvme_ctrl_is_persistent(c))
 			nvme_ctrl_set_persistent(c, true);
@@ -189,31 +192,34 @@ static void json_parse_host(nvme_root_t r, struct json_object *host_obj)
 	}
 }
 
+static DEFINE_CLEANUP_FUNC(cleanup_tokener, json_tokener *, json_tokener_free)
+#define _cleanup_tokener_ __cleanup__(cleanup_tokener)
+
 static struct json_object *parse_json(nvme_root_t r, int fd)
 {
 	char buf[JSON_FILE_BUF_SIZE];
-	struct json_object *obj = NULL;
+	struct json_object *obj;
 	char *str = NULL;
-	json_tokener *tok = NULL;
+	_cleanup_tokener_ json_tokener *tok = NULL;
 	int ret;
-	void *ptr = NULL;
+	_cleanup_free_ void *ptr = NULL;
 	int len = 0;
 
 	while ((ret = read(fd, buf, JSON_FILE_BUF_SIZE)) > 0) {
 		str = realloc(ptr, len + ret);
 		if (!str)
-			goto out;
+			return NULL;
 		memcpy(&str[len], buf, ret);
 		len += ret;
 		ptr = str;
 	}
 
 	if (ret < 0 || !len)
-		goto out;
+		return NULL;
 
 	tok = json_tokener_new_ex(JSON_TOKENER_DEFAULT_DEPTH);
 	if (!tok)
-		goto out;
+		return NULL;
 
 	/* Enforce correctly formatted JSON */
 	tok->flags = JSON_TOKENER_STRICT;
@@ -222,10 +228,6 @@ static struct json_object *parse_json(nvme_root_t r, int fd)
 	if (!obj)
 		nvme_msg(r, LOG_DEBUG, "JSON parsing failed: %s\n",
 			 json_util_get_last_err());
-out:
-	if (tok)
-		json_tokener_free(tok);
-	free(ptr);
 
 	return obj;
 }
@@ -325,6 +327,7 @@ static void json_update_port(struct json_object *ctrl_array, nvme_ctrl_t c)
 	JSON_BOOL_OPTION(cfg, port_obj, hdr_digest);
 	JSON_BOOL_OPTION(cfg, port_obj, data_digest);
 	JSON_BOOL_OPTION(cfg, port_obj, tls);
+	JSON_BOOL_OPTION(cfg, port_obj, concat);
 	if (nvme_ctrl_is_persistent(c))
 		json_object_object_add(port_obj, "persistent",
 				       json_object_new_boolean(true));
@@ -335,21 +338,21 @@ static void json_update_port(struct json_object *ctrl_array, nvme_ctrl_t c)
 	 * Store the keyring description in the JSON config file.
 	 */
 	if (cfg->keyring) {
-		char *desc = nvme_describe_key_serial(cfg->keyring);
+		_cleanup_free_ char *desc =
+			nvme_describe_key_serial(cfg->keyring);
 
 		if (desc) {
 			json_object_object_add(port_obj, "keyring",
 					       json_object_new_string(desc));
-			free(desc);
 		}
 	}
 	if (cfg->tls_key) {
-		char *desc = nvme_describe_key_serial(cfg->tls_key);
+		_cleanup_free_ char *desc =
+			nvme_describe_key_serial(cfg->tls_key);
 
 		if (desc) {
 			json_object_object_add(port_obj, "tls_key",
 					       json_object_new_string(desc));
-			free(desc);
 		}
 	}
 
@@ -501,6 +504,7 @@ static void json_dump_ctrl(struct json_object *ctrl_array, nvme_ctrl_t c)
 	JSON_BOOL_OPTION(cfg, ctrl_obj, hdr_digest);
 	JSON_BOOL_OPTION(cfg, ctrl_obj, data_digest);
 	JSON_BOOL_OPTION(cfg, ctrl_obj, tls);
+	JSON_BOOL_OPTION(cfg, ctrl_obj, concat);
 	if (nvme_ctrl_is_persistent(c))
 		json_object_object_add(ctrl_obj, "persistent",
 				       json_object_new_boolean(true));
@@ -572,9 +576,9 @@ int json_dump_tree(nvme_root_t r)
 	}
 	json_object_object_add(json_root, "hosts", host_array);
 
-	ret = json_object_to_fd(1, json_root, JSON_C_TO_STRING_PRETTY);
+	ret = json_object_to_fd(fileno(r->fp), json_root, JSON_C_TO_STRING_PRETTY);
 	if (ret < 0) {
-		nvme_msg(r, LOG_ERR, "Failed to write to stdout, %s\n",
+		nvme_msg(r, LOG_ERR, "Failed to write, %s\n",
 			 json_util_get_last_err());
 		ret = -1;
 		errno = EIO;
